#include <inttypes.h>
#include <avr/io.h>
#include <util/delay.h>

//This program samples all ADCx ports and prints the values on UART1
//It is designed to test the active 2-wire LEGO sensors module.
//Connect:
//   power-on-sensors to PD6 (T1), together with the GND 
//   the ADC1-4 signals from the light sensor module to any of the ADc0-ADC7 Cerebots inputs
//   VCC, and VU of the module to corresponding pins on the board
//   Bluesmirf module (or other serial device) to TXD1,RXD1

//The program uses AVRlib, and is written for WinAVR

#include <stdio.h>

#include <uart2.h>
#include <a2d.h>

#define POWER_SENSORS_ON PORTD = PIND | 64;
#define POWER_SENSORS_OFF PORTD = PIND & ~64;
	
static int uart1_putchar(char c, FILE *stream);
static FILE mystdout = FDEV_SETUP_STREAM(uart1_putchar, NULL, _FDEV_SETUP_WRITE);

// outputs one character to UART1
static int uart1_putchar(char c, FILE *stream)
{
 //we live in Windows world now 
 if (c == '\n') uart1_putchar('\r', stream);

 //use UCSRnA, UDREn, UDRn,  for n=0,1,2 for different UARTs
 loop_until_bit_is_set(UCSR1A, UDRE1);
 UDR1 = c;

 return 0;
}

// program entry point
int main(void)
{
  int i;
  uint16_t val1, val2, val3, val4, val5, val6, val7, val8;
  
  //Tx1/Rx1: serial port for console (2400, odd, 1)
  
  DDRA = 0x0;
  DDRB = 0x0;
  DDRC = 0x0;
  DDRD = 64;       //power on sensors
  DDRE = 16 + 64;  //and two diodes, all are outputs
  
  PORTA = 0x0;
  PORTB = 0x0;
  PORTC = 0x0;
  PORTD = 0x0;
  PORTE = 0x0;
  
  // indicate with LED that program runs
  PORTE = PINE | 16;   
  
  //initialize UART1 as 2400/odd parity (that's how our BlueSmirf is configured)
  uart1Init();
  uartSetBaudRate(1, 2400);
  UCSR1C |= 48;

  //setup character output function for stdout
  stdout = &mystdout;
  
  printf("Hello.\n");
    
  //setup ADC
  a2dInit();
  a2dSetPrescaler(ADC_PRESCALE_DIV128);
    
  //no pullups, we have our divider
  PORTA = 0x00;  
  
  //power the sensors
  POWER_SENSORS_ON;
  
  //run for a while
  for (i = 0; i < 30000; i++)
  {  
    //wait to charge sensor
    _delay_ms(1);
	//stop the power to sensors
    POWER_SENSORS_OFF;
	//wait for value to settle
	_delay_us(5);
	//read the value of ADC0
	val1 = a2dConvert10bit(0);
    //power the sensors again
	POWER_SENSORS_ON;

    //wait to charge sensor
    _delay_ms(1);
	//stop the power to sensors
    POWER_SENSORS_OFF;
	//wait for value to settle
	_delay_us(5);
	//read the value of ADC1
	val2 = a2dConvert10bit(1);
    //power the sensors again
	POWER_SENSORS_ON;

    //wait to charge sensor
    _delay_ms(1);
	//stop the power to sensors
    POWER_SENSORS_OFF;
	//wait for value to settle
	_delay_us(5);
	//read the value of ADC2
	val3 = a2dConvert10bit(2);
    //power the sensors again
	POWER_SENSORS_ON;

    //wait to charge sensor
    _delay_ms(1);
	//stop the power to sensors
    POWER_SENSORS_OFF;
	//wait for value to settle
	_delay_us(5);
	//read the value of ADC3
	val4 = a2dConvert10bit(3);
    //power the sensors again
	POWER_SENSORS_ON;

    //wait to charge sensor
    _delay_ms(1);
	//stop the power to sensors
    POWER_SENSORS_OFF;
	//wait for value to settle
	_delay_us(5);
	//read the value of ADC4
	val5 = a2dConvert10bit(4);
    //power the sensors again
	POWER_SENSORS_ON;

    //wait to charge sensor
    _delay_ms(1);
	//stop the power to sensors
    POWER_SENSORS_OFF;
	//wait for value to settle
	_delay_us(5);
	//read the value of ADC5
	val6 = a2dConvert10bit(5);
    //power the sensors again
	POWER_SENSORS_ON;

    //wait to charge sensor
    _delay_ms(1);
	//stop the power to sensors
    POWER_SENSORS_OFF;
	//wait for value to settle
	_delay_us(5);
	//read the value of ADC6
	val7 = a2dConvert10bit(6);
    //power the sensors again
	POWER_SENSORS_ON;

    //wait to charge sensor
    _delay_ms(1);
	//stop the power to sensors
    POWER_SENSORS_OFF;
	//wait for value to settle
	_delay_us(5);
	//read the value of ADC7
	val8 = a2dConvert10bit(7);
    //power the sensors again
	POWER_SENSORS_ON;

	//every once in a while print the values of all sensors
	if ((i & 127) == 64)
	{
      PORTE = PINE ^ 64;
	  printf("%d: %d, %d, %d, %d, %d, %d, %d, %d\n", i, val1, val2, val3, val4, val5, val6, val7, val8);
	  _delay_ms(200);_delay_ms(200);
	}
  }
    
  //turn off LEDs and turn off power to the sensors
  PORTE = 0x0;
  POWER_SENSORS_OFF;
  
  //disable ADC
  a2dOff();
  
  return 0;
  
} //end main function


